package fr.ippon.kata.numeral;

import java.util.Map.Entry;
import java.util.NavigableMap;
import java.util.Optional;
import java.util.TreeMap;
import java.util.function.Function;

public class Numerals {
  private static final NavigableMap<Integer, String> CONVERSIONS = buildConversions();

  public static String toRoman(int arabic) {
    return highestKnownConversion(arabic).map(romanRepresentation(arabic)).orElse("");
  }

  private static Optional<Entry<Integer, String>> highestKnownConversion(int arabic) {
    return Optional.ofNullable(CONVERSIONS.floorEntry(arabic));
  }

  private static Function<Entry<Integer, String>, String> romanRepresentation(int arabic) {
    return conversion -> conversion.getValue() + toRoman(arabic - conversion.getKey());
  }

  private static NavigableMap<Integer, String> buildConversions() {
    NavigableMap<Integer, String> conversions = new TreeMap<>();

    conversions.put(1, "I");
    conversions.put(4, "IV");
    conversions.put(5, "V");
    conversions.put(9, "IX");
    conversions.put(10, "X");

    return conversions;
  }
}
